package com.sun.management;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import com.sun.jmx.mbeanserver.Util;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import javax.management.MalformedObjectNameException;
import java.security.PrivilegedExceptionAction;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.StandardMBean;

public final class MissionControl extends StandardMBean implements MissionControlMXBean
{
    private static final ObjectName MBEAN_NAME;
    private MBeanServer server;
    
    public MissionControl() {
        super(MissionControlMXBean.class, true);
    }
    
    @Override
    public ObjectName preRegister(MBeanServer server, ObjectName objectName) throws Exception {
        this.server = server;
        return MissionControl.MBEAN_NAME;
    }
    
    @Override
    public void unregisterMBeans() {
        this.doPrivileged(new PrivilegedExceptionAction<Void>() {
            @Override
            public Void run() {
                MissionControl.FlightRecorderHelper.unregisterWithMBeanServer(MissionControl.this.server);
                return null;
            }
        });
    }
    
    @Override
    public void registerMBeans() {
        this.doPrivileged(new PrivilegedExceptionAction<Void>() {
            @Override
            public Void run() throws MalformedObjectNameException {
                if (!MissionControl.this.server.isRegistered(new ObjectName("com.oracle.jrockit:type=FlightRecorder"))) {
                    try {
                        MissionControl.FlightRecorderHelper.registerWithMBeanServer(MissionControl.this.server);
                    }
                    catch (IllegalStateException ex) {}
                }
                return null;
            }
        });
    }
    
    private void doPrivileged(PrivilegedExceptionAction<Void> action) {
        try {
            AccessController.doPrivileged(action);
        }
        catch (PrivilegedActionException ex) {}
    }
    
    static {
        MBEAN_NAME = Util.newObjectName("com.sun.management:type=MissionControl");
    }
    
    private static class FlightRecorderHelper
    {
        static final String MBEAN_NAME = "com.oracle.jrockit:type=FlightRecorder";
        private static final Class<?> FLIGHTRECORDER_CLASS;
        private static final Method REGISTERWITHMBEANSERVER_METHOD;
        private static final Method UNREGISTERWITHMBEANSERVER_METHOD;
        
        private static Class<?> getClass(String name) {
            try {
                return Class.forName(name, true, MissionControl.FlightRecorderHelper.class.getClassLoader());
            }
            catch (ClassNotFoundException cause) {
                throw new InternalError("jfr.jar missing?", cause);
            }
        }
        
        private static Method getMethod(Class<?> clazz, String name, Class<?>... parameterTypes) {
            try {
                return clazz.getMethod(name, parameterTypes);
            }
            catch (NoSuchMethodException cause) {
                throw new InternalError(cause);
            }
        }
        
        private static Object invokeStatic(Method method, Object... args) {
            try {
                return method.invoke(null, args);
            }
            catch (InvocationTargetException ex) {
                Throwable cause = ex.getCause();
                if ((cause instanceof RuntimeException)) {
                    throw (RuntimeException)cause;
                }
                if ((cause instanceof Error)) {
                    throw (Error)cause;
                }
                throw new InternalError(cause);
            }
            catch (IllegalAccessException cause2) {
                throw new InternalError(cause2);
            }
        }
        
        static void registerWithMBeanServer(MBeanServer mBeanServer) {
            invokeStatic(MissionControl.FlightRecorderHelper.REGISTERWITHMBEANSERVER_METHOD, new Object[] { mBeanServer });
        }
        
        static void unregisterWithMBeanServer(MBeanServer mBeanServer) {
            invokeStatic(MissionControl.FlightRecorderHelper.UNREGISTERWITHMBEANSERVER_METHOD, new Object[] { mBeanServer });
        }
        
        static {
            FLIGHTRECORDER_CLASS = getClass("com.oracle.jrockit.jfr.FlightRecorder");
            REGISTERWITHMBEANSERVER_METHOD = getMethod(MissionControl.FlightRecorderHelper.FLIGHTRECORDER_CLASS, "registerWithMBeanServer", new Class[] { MBeanServer.class });
            UNREGISTERWITHMBEANSERVER_METHOD = getMethod(MissionControl.FlightRecorderHelper.FLIGHTRECORDER_CLASS, "unregisterWithMBeanServer", new Class[] { MBeanServer.class });
        }
    }
}
